#ifndef DEBTAGS_ENVIRONMENT_H
#define DEBTAGS_ENVIRONMENT_H

#pragma interface

#include <Exception.h>
#include <TagcollConsumer.h>
#include <ParserBase.h>
#include <InputMerger.h>

namespace Debtags
{

class Environment
{
protected:
	static Environment* instance;

	Environment(bool editable) throw (SystemException, ConsistencyCheckException);

	// If true, save state to ~/.debtags
	bool editable;

	// State directory
	std::string rcdir;
	
public:
	static void init(bool editable = false) throw (SystemException, ConsistencyCheckException);
	static Environment& get() throw () { return *instance; }

	// Return the full path of the state storage directory
	const std::string& statedir() const throw () { return rcdir; }

	// Check if the tag database has been created (i.e. if debtags update has been run)
	bool hasTagDatabase() throw ();

	// Output the current Debian tags database to a TagcollConsumer
	// The collection is sent to 'cons' without merging repeated items
	void outputSystem(TagcollConsumer<std::string, std::string>& cons) const
		throw (FileException, ParserException);

	// Output the current Debian tags database, patched with local patch, to a TagcollConsumer
	void outputPatched(TagcollConsumer<std::string, std::string>& cons) const
		throw (FileException, ParserException);

	// Save a patch to turn the system database into the collection given
	void savePatch(const InputMerger<std::string>& collection)
		throw (FileException, ParserException, SystemException);

	// Send the current tag patch to the central archive
	void sendPatch(const InputMerger<std::string>& collection)
		throw (FileException, ParserException, SystemException, ConsistencyCheckException);
};

};

// vim:set ts=4 sw=4:
#endif
