#ifndef VOCABULARY_H
#define VOCABULARY_H

/*
 * Tag vocabulary access
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string>
#include <list>
#include <map>

#include <ParserBase.h>
#include <TagcollConsumer.h>
#include <DerivedTags.h>

class Tagexpr;

namespace Debtags
{

class Vocabulary
{
public:
	struct Record
	{
		std::list<std::string> implies;
		Tagexpr* derived;
		std::string description;
		std::list< std::pair<std::string, std::string> > others;

		Record() throw () : derived(0) {}
		void parseImplies(const std::string& impls) throw (ParserException);
		std::string formatImplies() const throw ();
	};

protected:
	std::map<std::string, Record> contents;
public:
	typedef std::map<std::string, Record>::iterator iterator;
	typedef std::map<std::string, Record>::const_iterator const_iterator;

	const_iterator begin() const throw () { return contents.begin(); }
	const_iterator end() const throw () { return contents.end(); }
	const_iterator find(const std::string& key) const throw () { return contents.find(key); }

	iterator begin() throw () { return contents.begin(); }
	iterator end() throw () { return contents.end(); }
	iterator find(const std::string& key) throw () { return contents.find(key); }
	
	const Record& operator[](const std::string tag) throw ()
	{
		return contents[tag];
	}

	bool has(const std::string& tag) const throw ()
	{
		return contents.find(tag) != contents.end();
	}

	void read(ParserInput& input) throw (ParserException);
	void read() throw (FileException, ParserException);
	void readVocabularies() throw (SystemException, FileException, ParserException);

	void write() throw (FileException);
	void writeImplications() throw (FileException);
	void writeDerivedTags() throw (FileException);

	void outputImplications(TagcollConsumer<std::string>& consumer) const throw ();
	void outputDerivedTags(DerivedTagList& dtlist) const throw ();
};

};

// vim:set ts=4 sw=4:
#endif
